@@ -9,9 +9,10 @@
 
 
 from typing import Unpack
+
 from synnax.cli.console.sugared import AskKwargs
-from synnax.telem import DataType, TimeSpan
 from synnax.cli.flow import Context
+from synnax.telem import DataType, TimeSpan, TimeSpanUnits
 
 
 def select_data_type(
@@ -38,7 +39,7 @@ def ask_time_units_select(
     ctx: Context,
     question: str | None = None,
     **kwargs: Unpack[AskKwargs[str]],
-) -> str:
+) -> TimeSpanUnits:
     """Prompts the user to select a time unit from a list of all available time
     units.
 
@@ -48,7 +49,7 @@ def ask_time_units_select(
     if question is not None:
         ctx.console.info(question)
     return ctx.console.select(
-        rows=list(TimeSpan.UNITS.keys()),
+        rows=["iso", *list(TimeSpan.UNITS.keys())],
         type_=str,
         columns=["unit"],
         **kwargs,
