@@ -10,8 +10,8 @@
 import keyring
 from pydantic import BaseModel
 
-from synnax.options import SynnaxOptions
 from synnax.config.file import ConfigFile
+from synnax.options import SynnaxOptions
 
 
 class ClusterConfig(BaseModel):
@@ -27,10 +27,10 @@ class ClustersConfig:
     def get(self, key: str = "default") -> ClusterConfig | None:
         c = self.internal.get(f"clusters.{key}")
         if c is None:
-            return
+            return None
         opts = c["options"]
         pwd = keyring.get_password("synnax", key)
-        pwd = "" if pwd is None else pwd
+        pwd = pwd or ""
         return ClusterConfig(options=SynnaxOptions(**opts, password=pwd))
 
     def set(self, c: ClusterConfig, key: str = "default"):
