@@ -7,34 +7,38 @@
 #  License, use of this software will be governed by the Apache License, Version 2.0,
 #  included in the file licenses/APL.txt.
 
-from urllib3 import Timeout, Retry
-
+from alamos import Instrumentation
 from freighter import (
     URL,
     AsyncMiddleware,
     AsyncStreamClient,
-    HTTPClientPool,
+    HTTPClient,
     JSONEncoder,
     Middleware,
     MsgpackEncoder,
     StreamClient,
     SyncStreamClient,
+    UnaryClient,
     WebsocketClient,
+    async_instrumentation_middleware,
+    instrumentation_middleware,
 )
+from urllib3 import Retry, Timeout
 
-from synnax.telem import TimeSpan
+from synnax.telem import Size, TimeSpan
 
 
 class Transport:
     url: URL
     stream: StreamClient
     stream_async: AsyncStreamClient
-    http: HTTPClientPool
+    unary: UnaryClient
     secure: bool
 
     def __init__(
         self,
         url: URL,
+        instrumentation: Instrumentation,
         secure: bool = False,
         open_timeout: TimeSpan = TimeSpan.SECOND * 5,
         read_timeout: TimeSpan = TimeSpan.SECOND * 5,
@@ -45,26 +49,26 @@ class Transport:
         self.stream_async = WebsocketClient(
             base_url=self.url,
             encoder=MsgpackEncoder(),
-            max_message_size=int(5e6),
+            max_message_size=int(Size.MB * 5),
             secure=secure,
-            open_timeout=open_timeout.seconds(),
-            ping_interval=keep_alive.seconds(),
-            close_timeout=read_timeout.seconds(),
-            ping_timeout=read_timeout.seconds(),
+            open_timeout=open_timeout.seconds,
+            ping_interval=keep_alive.seconds,
+            close_timeout=read_timeout.seconds,
+            ping_timeout=180,
         )
         self.stream = SyncStreamClient(self.stream_async)
-        self.http = HTTPClientPool(
+        self.unary = HTTPClient(
             url=self.url,
             encoder_decoder=JSONEncoder(),
             secure=secure,
-            timeout=Timeout(
-                connect=open_timeout.seconds(), read=read_timeout.seconds()
-            ),
+            timeout=Timeout(connect=open_timeout.seconds, read=read_timeout.seconds),
             retries=Retry(total=max_retries),
         )
+        self.use(instrumentation_middleware(instrumentation))
+        self.use_async(async_instrumentation_middleware(instrumentation))
 
     def use(self, *middleware: Middleware):
-        self.http.use(*middleware)
+        self.unary.use(*middleware)
         self.stream.use(*middleware)
 
     def use_async(self, *middleware: AsyncMiddleware):
